//
// Created by h44z on 07.05.19.
//

#include "GmcDevice.h"

GmcDevice::GmcDevice(const string& serialPort, int baud) {
    device = new SerialPort(serialPort, baud);

    if(!device->isOpen()) {
        cout << "Failed to open gmc device!" << endl;
    } else {
        cout << "Connection to gmc device established!" << endl;
        if(!setHeartbeatOff()) {
            cout << "Failed to disable heartbeat!" << endl;
            device->serialClose();
        }
    }
}

GmcDevice::~GmcDevice() {
    close();
}

bool GmcDevice::close() {
    return device->serialClose();
}

int GmcDevice::getCPM() {
    if(!device->isOpen()) {
        cout << "Device is not connected, failed to read CPM!" << endl;
        return -1;
    }

    string cmd = "<GETCPM>>";
    string result;

    if(device->serialWrite(cmd) == cmd.length()) {
        device->serialRead(result, 2); // cpm result has size 2
    }

    return result[0] * 256 + result[1];
}

float GmcDevice::getTemperature() {
    if(!device->isOpen()) {
        cout << "Device is not connected, failed to read temperature!" << endl;
        return -1;
    }

    string cmd = "<GETTEMP>>";
    string result;

    if(device->serialWrite(cmd) == cmd.length()) {
        device->serialRead(result, 4); // temp result has size 4
    }

    int sign =  result[2] == 0 ? 1 : -1;
    float temp = result[0];
    temp += (float) result[1] / 10;
    temp = temp * sign;
    return temp;
}

string GmcDevice::getVersion() {
    if(!device->isOpen()) {
        cout << "Device is not connected, failed to read version!" << endl;
        return "";
    }

    string cmd = "<GETVER>>";
    string result;

    if(device->serialWrite(cmd) == cmd.length()) {
        device->serialRead(result, 14); // version result has size 14
    }

    return result;
}

bool GmcDevice::setHeartbeatOff() {
    if(!device->isOpen()) {
        cout << "Device is not connected, failed to disable heartbeat!" << endl;
        return false;
    }

    return false;
}

bool GmcDevice::isConnected() {
    return device->isOpen();
}
